package meowlib_test

import (
	"encoding/json"
	"fmt"
	"os"
	"testing"

	"forge.redroom.link/yves/meowlib"
	"forge.redroom.link/yves/meowlib/client"
	"forge.redroom.link/yves/meowlib/server"
)

func TestEndToEnd(t *testing.T) {
	//
	// Create my own identity
	//
	fmt.Println("Trying to load identity from file.")
	Me, err := client.LoadIdentity("id.enc", "Test")
	if err != nil {
		fmt.Println("Failed : creating New identity...")
		///////////////////////////
		// Creating New Identity //
		///////////////////////////
		Me = client.CreateIdentity("myname")
		// define my preferences (servers)
		Me.MessageServers.Name = "Message Servers"
		Me.MessageServers.AddUrls([]string{"http://127.0.0.1/meow/", "mqtt://127.0.0.1", "meow://127.0.0.1"})
		////////////////////////////////////////////////////////////////////////////
		// Create an invitation for a friend, I want him/her to know me as Bender //
		////////////////////////////////////////////////////////////////////////////
		fmt.Println("Creating an invitation for the first friend...")
		peer, myContactCard, err := Me.InvitePeer("Bender", "myfirstfriend", []int{1, 2})
		if err != nil {
			println(err)
		}
		println(peer.Name)
		// print my invitation
		a, _ := json.Marshal(myContactCard)
		fmt.Println(string(a))
		// TODO : Convert invitation to QR Code
		myContactCard.WritePng("invitation.png")
		data, err := myContactCard.Compress()
		if err != nil {
			println(err)
		}
		myContactCard.WriteQr("qrcode.png")
		println("Compressed contact card :", len(data))
		///////////////////////////////////////
		// Simulate peer invitation response //
		///////////////////////////////////////
		fmt.Println("Simulating first friend answer...")
		var ReceivedContact meowlib.ContactCard

		// Friend simulated invitation
		FirstFriendContactKp := meowlib.NewKeyPair()
		FirstFriendEncryptionKp := meowlib.NewKeyPair()
		FirstFriendLookupKp := meowlib.NewKeyPair()
		ReceivedContact.Name = "I'm the friend"
		ReceivedContact.ContactPublicKey = FirstFriendContactKp.Public
		ReceivedContact.EncryptionPublicKey = FirstFriendEncryptionKp.Public
		ReceivedContact.LookupPublicKey = FirstFriendLookupKp.Public
		ReceivedContact.InvitationId = myContactCard.InvitationId
		FriendServer1KP := meowlib.NewKeyPair()
		FriendServer1 := meowlib.ServerCard{Name: "FriendServer1", Url: "http://myfriend.org/meow/", PublicKey: FriendServer1KP.Public, Description: "Fancy description"}
		ReceivedContact.PullServers = append(ReceivedContact.PullServers, &FriendServer1)

		///////////////////////////////////////////////////////
		// Finalize the contact with the invitation response //
		///////////////////////////////////////////////////////
		Me.FinalizeInvitation(&ReceivedContact)
		err = Me.Save()
		if err != nil {
			fmt.Println(err.Error())
		}

		a, _ = json.Marshal(Me)
		os.WriteFile("id.json", a, 0644)
		fmt.Println(string(a))
		/////////////////////////////////////
		// Create a message to that friend //
		/////////////////////////////////////
		MyFirstFriend := Me.Peers[0]
		textmessage := "Hello friend!"
		// Creating User message
		usermessage, err := MyFirstFriend.BuildSimpleUserMessage([]byte(textmessage))
		if err != nil {
			fmt.Println(err.Error())
		}
		serializedMessage, err := MyFirstFriend.SerializeUserMessage(usermessage)
		if err != nil {
			fmt.Println(err.Error())
		}
		// Encrypting it
		enc, err := MyFirstFriend.AsymEncryptMessage(serializedMessage)
		if err != nil {
			fmt.Println(err.Error())
		}

		// Packing it
		packedMsg := MyFirstFriend.PackUserMessage(enc.Data, enc.Signature)

		srv := MyFirstFriend.Contact.PullServers[0]
		intS1 := client.CreateServerFromServerCard(srv)

		// Creating Server message for transporting the user message
		toServerMessage, err := intS1.BuildMessageSendingMessage(packedMsg)
		if err != nil {
			fmt.Println(err.Error())
		}
		// Encrypting it
		encToServer, err := intS1.AsymEncryptMessage(toServerMessage)
		if err != nil {
			fmt.Println(err.Error())
		}
		// Packing it
		protoPackedServerMsg, err := intS1.PackServerMessage(encToServer.Data, encToServer.Signature)
		if err != nil {
			fmt.Println(err.Error())
		}
		///////////////////////
		// Sending to server //
		///////////////////////
		//=> fake action (network protocol transfer are not part of meowlib)
		// You have to implement the netwok layer
		// Just FYI printing final byte array size. Those bytes will be sent over the network.
		println(len(protoPackedServerMsg))

		///////////////////////////////////////
		// Simulating server side processing //
		///////////////////////////////////////
		var server1 server.Identity
		server1.ServerName = intS1.ServerData.Name
		server1.ServerKp = FriendServer1KP
		server1.ServerDesc = intS1.ServerData.Description
		// Unpack
		srv_from, srv_encmsg, srv_signature, err := server1.UnpackReceived(protoPackedServerMsg)
		if err != nil {
			fmt.Println(err.Error())
		}
		// Decrypt
		srv_clear, err := server1.AsymDecryptMessage(srv_from, srv_encmsg, srv_signature)
		if err != nil {
			fmt.Println(err.Error())
		}
		// Decode msg
		srv_msg, err := server1.DeserializeToServerMessage(srv_clear)
		if err != nil {
			fmt.Println(err.Error())
		}
		// Response : Ack received message
		srv_fromServerMessage, err := server1.BuildSimpleAckResponseMessage(srv_msg.Uuid)
		if err != nil {
			fmt.Println(err.Error())
		}
		encoded_srv_fromServerMessage, err := server1.SerializeFromServerMessage(srv_fromServerMessage)
		if err != nil {
			fmt.Println(err.Error())
		}
		srv_resp, err := server1.AsymEncryptMessage(srv_from, encoded_srv_fromServerMessage)
		if err != nil {
			fmt.Println(err.Error())
		}
		resp, err := server1.PackForSending(srv_resp.Data, srv_resp.Signature)
		if err != nil {
			fmt.Println(err.Error())
		}
		print(resp)
		//////////////////////////////////////////////
		// Back to client, decoding server response //
		//////////////////////////////////////////////

		// server unpack

		// server decrypt

		// server decode protobuf

		// user unpack

		// user decrypt
		/*decMess, err2 := MyFirstFriend.AsymDecryptMessage([]byte(enc.Data), enc.Signature)
		if err2 != nil {
			fmt.Println(err2.Error())
		}
		fmt.Println(decMess)*/
		// user decode protobuf

	}

}
